@@ -1,19 +1,9 @@
 # -*- coding: utf-8 -*-
 ##	config.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##	- Stéphan Kochen <stephan@kochen.nl>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2005-2006 Stéphan Kochen <stephan@kochen.nl>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -36,10 +26,10 @@
 # - def update_actions(self)
 # - def default_action(self)
 # - def _find_item(self, jid, node)
-# - def _add_item(self, model, jid, node, item, force)
-# - def _update_item(self, model, iter, jid, node, item)
-# - def _update_info(self, model, iter, jid, node, identities, features, data)
-# - def _update_error(self, model, iter, jid, node)
+# - def _add_item(self, jid, node, item, force)
+# - def _update_item(self, iter, jid, node, item)
+# - def _update_info(self, iter, jid, node, identities, features, data)
+# - def _update_error(self, iter, jid, node)
 #
 # * Should call the super class for this method.
 # All others do not have to call back to the super class. (but can if they want
@@ -51,23 +41,16 @@ import inspect
 import weakref
 import gobject
 import gtk
-import gtk.glade
 import pango
 
 import dialogs
 import tooltips
+import gtkgui_helpers
+import groups
 
 from common import gajim
 from common import xmpp
-from common import i18n
-
-_ = i18n._
-APP = i18n.APP
-gtk.glade.bindtextdomain (APP, i18n.DIR)
-gtk.glade.textdomain (APP)
-
-GTKGUI_GLADE = 'gtkgui.glade'
-
+from common.exceptions import GajimGeneralException
 
 # Dictionary mapping category, type pairs to browser class, image pairs.
 # This is a function, so we can call it after the classes are declared.
@@ -91,10 +74,11 @@ def _gen_agent_type_info():
 		('gateway', 'sip'):			(False, 'sip.png'),
 		('directory', 'user'):		(None, 'jud.png'),
 		('pubsub', 'generic'):		(None, 'pubsub.png'),
+		('pubsub', 'service'):		(PubSubBrowser, 'pubsub.png'),
 		('proxy', 'bytestreams'):	(None, 'bytestreams.png'), # Socks5 FT proxy
 
 		# Transports
-		('conference', 'irc'):		(False, 'irc.png'),
+		('conference', 'irc'):		(ToplevelAgentBrowser, 'irc.png'),
 		('_jid', 'irc'):				(False, 'irc.png'),
 		('gateway', 'aim'):			(False, 'aim.png'),
 		('_jid', 'aim'):				(False, 'aim.png'),
@@ -140,9 +124,17 @@ class CacheDictionary:
 		def __call__(self):
 			return self.value
 
+	def cleanup(self):
+		for key in self.cache.keys():
+			item = self.cache[key]
+			if item.source:
+				gobject.source_remove(item.source)
+			del self.cache[key]
+
 	def _expire_timeout(self, key):
 		'''The timeout has expired, remove the object.'''
-		del self.cache[key]
+		if key in self.cache:
+			del self.cache[key]
 		return False
 
 	def _refresh_timeout(self, key):
@@ -150,8 +142,9 @@ class CacheDictionary:
 		item = self.cache[key]
 		if item.source:
 			gobject.source_remove(item.source)
-		source = gobject.timeout_add(self.lifetime, self._expire_timeout, key)
-		item.source = source
+		if self.lifetime:
+			source = gobject.timeout_add(self.lifetime, self._expire_timeout, key)
+			item.source = source
 
 	def __getitem__(self, key):
 		item = self.cache[key]
@@ -223,10 +216,15 @@ class ServicesCache:
 	ServiceCache instance.'''
 	def __init__(self, account):
 		self.account = account
-		self._items = CacheDictionary(15, getrefresh = False)
-		self._info = CacheDictionary(15, getrefresh = False)
+		self._items = CacheDictionary(0, getrefresh = False)
+		self._info = CacheDictionary(0, getrefresh = False)
+		self._subscriptions = CacheDictionary(5, getrefresh=False)
 		self._cbs = {}
 
+	def cleanup(self):
+		self._items.cleanup()
+		self._info.cleanup()
+
 	def _clean_closure(self, cb, type, addr):
 		# A closure died, clean up
 		cbkey = (type, addr)
@@ -280,7 +278,7 @@ class ServicesCache:
 			except KeyError:
 				continue
 			browser = info[0]
-			if browser is not None:
+			if browser:
 				break
 		# Note: possible outcome here is browser=False
 		if browser is None:
@@ -398,12 +396,12 @@ class ServicesCache:
 			if self._cbs.has_key(cbkey):
 				del self._cbs[cbkey]
 
-# object is needed so that property() works
+# object is needed so that @property works
 class ServiceDiscoveryWindow(object):
 	'''Class that represents the Services Discovery window.'''
 	def __init__(self, account, jid = '', node = '',
 			address_entry = False, parent = None):
-		self._account = account
+		self.account = account
 		self.parent = parent
 		if not jid:
 			jid = gajim.config.get_per('accounts', account, 'hostname')
@@ -427,9 +425,10 @@ _('Without a connection, you can not browse available services'))
 			self.cache = ServicesCache(account)
 			gajim.connections[account].services_cache = self.cache
 
-		self.xml = gtk.glade.XML(GTKGUI_GLADE, 'service_discovery_window', APP)
+		self.xml = gtkgui_helpers.get_glade('service_discovery_window.glade')
 		self.window = self.xml.get_widget('service_discovery_window')
 		self.services_treeview = self.xml.get_widget('services_treeview')
+		self.model = None
 		# This is more reliable than the cursor-changed signal.
 		selection = self.services_treeview.get_selection()
 		selection.connect_after('changed',
@@ -451,7 +450,7 @@ _('Without a connection, you can not browse available services'))
 
 		# Address combobox
 		self.address_comboboxentry = None
-		address_hbox = self.xml.get_widget('address_hbox')
+		address_table = self.xml.get_widget('address_table')
 		if address_entry:
 			self.address_comboboxentry = self.xml.get_widget(
 				'address_comboboxentry')
@@ -460,10 +459,8 @@ _('Without a connection, you can not browse available services'))
 
 			liststore = gtk.ListStore(str)
 			self.address_comboboxentry.set_model(liststore)
-			self.address_comboboxentry.set_text_column(0)
 			self.latest_addresses = gajim.config.get(
 				'latest_disco_addresses').split()
-			jid = gajim.get_hostname_from_account(self.account)
 			if jid in self.latest_addresses:
 				self.latest_addresses.remove(jid)
 			self.latest_addresses.insert(0, jid)
@@ -474,38 +471,43 @@ _('Without a connection, you can not browse available services'))
 			self.address_comboboxentry.child.set_text(jid)
 		else:
 			# Don't show it at all if we didn't ask for it
-			address_hbox.set_no_show_all(True)
-			address_hbox.hide()
+			address_table.set_no_show_all(True)
+			address_table.hide()
 
 		self._initial_state()
 		self.xml.signal_autoconnect(self)
 		self.travel(jid, node)
 		self.window.show_all()
 
+	@property
 	def _get_account(self):
-		return self._account
+		return self.account
 
+	@property
 	def _set_account(self, value):
-		self._account = value
+		self.account = value
 		self.cache.account = value
 		if self.browser:
 			self.browser.account = value
-	account = property(_get_account, _set_account)
 
 	def _initial_state(self):
 		'''Set some initial state on the window. Separated in a method because
 		it's handy to use within browser's cleanup method.'''
 		self.progressbar.hide()
-		self.window.set_title(_('Service Discovery using account %s') % self.account)
+		title_text = _('Service Discovery using account %s') % self.account
+		self.window.set_title(title_text)
 		self._set_window_banner_text(_('Service Discovery'))
-		# FIXME: use self.banner_icon.clear() when we switch to GTK 2.8
-		self.banner_icon.set_from_file(None)
-		self.banner_icon.hide()		# Just clearing it doesn't work
+		if gtk.gtk_version >= (2, 8, 0) and gtk.pygtk_version >= (2, 8, 0):
+			self.banner_icon.clear()
+		else:
+			self.banner_icon.set_from_file(None)
+		self.banner_icon.hide() # Just clearing it doesn't work
 
 	def _set_window_banner_text(self, text, text_after = None):
 		theme = gajim.config.get('roster_theme')
 		bannerfont = gajim.config.get_per('themes', theme, 'bannerfont')
-		bannerfontattrs = gajim.config.get_per('themes', theme, 'bannerfontattrs')
+		bannerfontattrs = gajim.config.get_per('themes', theme,
+			'bannerfontattrs')
 		
 		if bannerfont:
 			font = pango.FontDescription(bannerfont)
@@ -581,7 +583,7 @@ _('Without a connection, you can not browse available services'))
 		self.connect_style_event(opts[0], opts[1])
 	
 	def destroy(self, chain = False):
-		'''Close the browser. This can optionally close it's children and
+		'''Close the browser. This can optionally close its children and
 		propagate to the parent. This should happen on actions like register,
 		or join to kill off the entire browser chain.'''
 		if self.dying:
@@ -604,10 +606,13 @@ _('Without a connection, you can not browse available services'))
 				child.destroy(chain = chain)
 				self.children.remove(child)
 		if self.parent:
-			self.parent.children.remove(self)
+			if self in self.parent.children:
+				self.parent.children.remove(self)
 			if chain and not self.parent.children:
 				self.parent.destroy(chain = chain)
 				self.parent = None
+		else:
+			self.cache.cleanup()
 
 	def travel(self, jid, node):
 		'''Travel to an agent within the current services window.'''
@@ -729,9 +734,9 @@ class AgentBrowser:
 		note that the first two columns should ALWAYS be of type string and
 		contain the JID and node of the item respectively.'''
 		# JID, node, name, address
-		model = gtk.ListStore(str, str, str, str)
-		model.set_sort_column_id(3, gtk.SORT_ASCENDING)
-		self.window.services_treeview.set_model(model)
+		self.model = gtk.ListStore(str, str, str, str)
+		self.model.set_sort_column_id(3, gtk.SORT_ASCENDING)
+		self.window.services_treeview.set_model(self.model)
 		# Name column
 		col = gtk.TreeViewColumn(_('Name'))
 		renderer = gtk.CellRendererText()
@@ -749,7 +754,7 @@ class AgentBrowser:
 		self.window.services_treeview.set_headers_visible(True)
 
 	def _clean_treemodel(self):
-		self.window.services_treeview.get_model().clear()
+		self.model.clear()
 		for col in self.window.services_treeview.get_columns():
 			self.window.services_treeview.remove_column(col)
 		self.window.services_treeview.set_headers_visible(False)
@@ -881,8 +886,7 @@ class AgentBrowser:
 
 	def browse(self, force = False):
 		'''Fill the treeview with agents, fetching the info if necessary.'''
-		model = self.window.services_treeview.get_model()
-		model.clear()
+		self.model.clear()
 		self._total_items = self._progress = 0
 		self.window.progressbar.show()
 		self._pulse_timeout = gobject.timeout_add(250, self._pulse_timeout_cb)
@@ -899,21 +903,21 @@ class AgentBrowser:
 	def _find_item(self, jid, node):
 		'''Check if an item is already in the treeview. Return an iter to it
 		if so, None otherwise.'''
-		model = self.window.services_treeview.get_model()
-		iter = model.get_iter_root()
+		iter = self.model.get_iter_root()
 		while iter:
-			cjid = model.get_value(iter, 0).decode('utf-8')
-			cnode = model.get_value(iter, 1).decode('utf-8')
+			cjid = self.model.get_value(iter, 0).decode('utf-8')
+			cnode = self.model.get_value(iter, 1).decode('utf-8')
 			if jid == cjid and node == cnode:
 				break
-			iter = model.iter_next(iter)
+			iter = self.model.iter_next(iter)
 		if iter:
 			return iter
 		return None
 
 	def _agent_items(self, jid, node, items, force):
 		'''Callback for when we receive a list of agent items.'''
-		model = self.window.services_treeview.get_model()
+		self.model.clear()
+		self._total_items = 0
 		gobject.source_remove(self._pulse_timeout)
 		self.window.progressbar.hide()
 		# The server returned an error
@@ -925,53 +929,48 @@ class AgentBrowser:
 _('This service does not contain any items to browse.'))
 			return
 		# We got a list of items
+		self.window.services_treeview.set_model(None)
 		for item in items:
 			jid = item['jid']
 			node = item.get('node', '')
-			iter = self._find_item(jid, node)
-			if iter:
-				# Already in the treeview
-				self._update_item(model, iter, jid, node, item)
-			else:
-				# Not in the treeview
-				self._total_items += 1
-				self._add_item(model, jid, node, item, force)
+			self._total_items += 1
+			self._add_item(jid, node, item, force)
+		self.window.services_treeview.set_model(self.model)
 
 	def _agent_info(self, jid, node, identities, features, data):
 		'''Callback for when we receive info about an agent's item.'''
 		addr = get_agent_address(jid, node)
-		model = self.window.services_treeview.get_model()
 		iter = self._find_item(jid, node)
 		if not iter:
 			# Not in the treeview, stop
 			return
 		if identities == 0:
 			# The server returned an error
-			self._update_error(model, iter, jid, node)
+			self._update_error(iter, jid, node)
 		else:
 			# We got our info
-			self._update_info(model, iter, jid, node,
+			self._update_info(iter, jid, node,
 				identities, features, data)
 		self.update_actions()
 
-	def _add_item(self, model, jid, node, item, force):
+	def _add_item(self, jid, node, item, force):
 		'''Called when an item should be added to the model. The result of a
 		disco#items query.'''
-		model.append((jid, node, item.get('name', ''),
+		self.model.append((jid, node, item.get('name', ''),
 			get_agent_address(jid, node)))
 
-	def _update_item(self, model, iter, jid, node, item):
+	def _update_item(self, iter, jid, node, item):
 		'''Called when an item should be updated in the model. The result of a
 		disco#items query. (seldom)'''
 		if item.has_key('name'):
-			model[iter][2] = item['name']
+			self.model[iter][2] = item['name']
 
-	def _update_info(self, model, iter, jid, node, identities, features, data):
+	def _update_info(self, iter, jid, node, identities, features, data):
 		'''Called when an item should be updated in the model with further info.
 		The result of a disco#info query.'''
-		model[iter][2] = identities[0].get('name', '')
+		self.model[iter][2] = identities[0].get('name', '')
 
-	def _update_error(self, model, iter, jid, node):
+	def _update_error(self, iter, jid, node):
 		'''Called when a disco#info query failed for an item.'''
 		pass
 
@@ -1055,14 +1054,12 @@ class ToplevelAgentBrowser(AgentBrowser):
 
 	# These are all callbacks to make tooltips work
 	def on_treeview_leave_notify_event(self, widget, event):
-		model = widget.get_model()
 		props = widget.get_path_at_pos(int(event.x), int(event.y))
 		if self.tooltip.timeout > 0:
 			if not props or self.tooltip.id == props[0]:
 				self.tooltip.hide_tooltip()
 
 	def on_treeview_motion_notify_event(self, widget, event):
-		model = widget.get_model()
 		props = widget.get_path_at_pos(int(event.x), int(event.y))
 		if self.tooltip.timeout > 0:
 			if not props or self.tooltip.id != props[0]:
@@ -1071,12 +1068,12 @@ class ToplevelAgentBrowser(AgentBrowser):
 			[row, col, x, y] = props
 			iter = None
 			try:
-				iter = model.get_iter(row)
+				iter = self.model.get_iter(row)
 			except:
 				self.tooltip.hide_tooltip()
 				return
-			jid = model[iter][0]
-			state = model[iter][4]
+			jid = self.model[iter][0]
+			state = self.model[iter][4]
 			# Not a category, and we have something to say about state
 			if jid and state > 0 and \
 					(self.tooltip.timeout == 0 or self.tooltip.id != props[0]):
@@ -1093,10 +1090,10 @@ class ToplevelAgentBrowser(AgentBrowser):
 		# JID, node, icon, description, state
 		# State means 2 when error, 1 when fetching, 0 when succes.
 		view = self.window.services_treeview
-		model = gtk.TreeStore(str, str, gtk.gdk.Pixbuf, str, int)
-		model.set_sort_func(4, self._treemodel_sort_func)
-		model.set_sort_column_id(4, gtk.SORT_ASCENDING)
-		view.set_model(model)
+		self.model = gtk.TreeStore(str, str, gtk.gdk.Pixbuf, str, int)
+		self.model.set_sort_func(4, self._treemodel_sort_func)
+		self.model.set_sort_column_id(4, gtk.SORT_ASCENDING)
+		view.set_model(self.model)
 
 		col = gtk.TreeViewColumn()
 		# Icon Renderer
@@ -1200,14 +1197,11 @@ class ToplevelAgentBrowser(AgentBrowser):
 		if not iter:
 			return
 		service = model[iter][0].decode('utf-8')
-		if service.find('@') != -1:
-			services = service.split('@', 1)
-			room = services[0]
-			service = services[1]
-		else:
-			room = ''
 		if not gajim.interface.instances[self.account].has_key('join_gc'):
-			dialogs.JoinGroupchatWindow(self.account, service, room)
+			try:
+				dialogs.JoinGroupchatWindow(self.account, service)
+			except GajimGeneralException:
+				pass
 		else:
 			gajim.interface.instances[self.account]['join_gc'].window.present()
 		self.window.destroy(chain = True)
@@ -1253,10 +1247,11 @@ class ToplevelAgentBrowser(AgentBrowser):
 			# We can register this agent
 			registered_transports = []
 			jid_list = gajim.contacts.get_jid_list(self.account)
-			for j in jid_list:
-				contact = gajim.contacts.get_first_contact_from_jid(self.account, j)
+			for jid in jid_list:
+				contact = gajim.contacts.get_first_contact_from_jid(
+					self.account, jid)
 				if _('Transports') in contact.groups:
-					registered_transports.append(j)
+					registered_transports.append(jid)
 			if jid in registered_transports:
 				self.register_button.set_label(_('_Edit'))
 			else:
@@ -1335,41 +1330,38 @@ class ToplevelAgentBrowser(AgentBrowser):
 
 	def _create_category(self, cat, type=None):
 		'''Creates a category row.'''
-		model = self.window.services_treeview.get_model()
 		cat, prio = self._friendly_category(cat, type)
-		return model.append(None, ('', '', None, cat, prio))
+		return self.model.append(None, ('', '', None, cat, prio))
 
 	def _find_category(self, cat, type=None):
 		'''Looks up a category row and returns the iterator to it, or None.'''
-		model = self.window.services_treeview.get_model()
 		cat, prio = self._friendly_category(cat, type)
-		iter = model.get_iter_root()
+		iter = self.model.get_iter_root()
 		while iter:
-			if model.get_value(iter, 3).decode('utf-8') == cat:
+			if self.model.get_value(iter, 3).decode('utf-8') == cat:
 				break
-			iter = model.iter_next(iter)
+			iter = self.model.iter_next(iter)
 		if iter:
 			return iter
 		return None
 
 	def _find_item(self, jid, node):
-		model = self.window.services_treeview.get_model()
 		iter = None
-		cat_iter = model.get_iter_root()
+		cat_iter = self.model.get_iter_root()
 		while cat_iter and not iter:
-			iter = model.iter_children(cat_iter)
+			iter = self.model.iter_children(cat_iter)
 			while iter:
-				cjid = model.get_value(iter, 0).decode('utf-8')
-				cnode = model.get_value(iter, 1).decode('utf-8')
+				cjid = self.model.get_value(iter, 0).decode('utf-8')
+				cnode = self.model.get_value(iter, 1).decode('utf-8')
 				if jid == cjid and node == cnode:
 					break
-				iter = model.iter_next(iter)
-			cat_iter = model.iter_next(cat_iter)
+				iter = self.model.iter_next(iter)
+			cat_iter = self.model.iter_next(cat_iter)
 		if iter:
 			return iter
 		return None
 
-	def _add_item(self, model, jid, node, item, force):
+	def _add_item(self, jid, node, item, force):
 		# Row text
 		addr = get_agent_address(jid, node)
 		if item.has_key('name'):
@@ -1393,21 +1385,21 @@ class ToplevelAgentBrowser(AgentBrowser):
 		cat = self._find_category(*cat_args)
 		if not cat:
 			cat = self._create_category(*cat_args)
-		model.append(cat, (item['jid'], item.get('node', ''), pix, descr, 1))
+		self.model.append(cat, (item['jid'], item.get('node', ''), pix, descr, 1))
 		self._expand_all()
 		# Grab info on the service
 		self.cache.get_info(jid, node, self._agent_info, force = force)
 		self._update_progressbar()
 
-	def _update_item(self, model, iter, jid, node, item):
+	def _update_item(self, iter, jid, node, item):
 		addr = get_agent_address(jid, node)
 		if item.has_key('name'):
 			descr = "<b>%s</b>\n%s" % (item['name'], addr)
 		else:
 			descr = "<b>%s</b>" % addr
-		model[iter][3] = descr
+		self.model[iter][3] = descr
 
-	def _update_info(self, model, iter, jid, node, identities, features, data):
+	def _update_info(self, iter, jid, node, identities, features, data):
 		addr = get_agent_address(jid, node)
 		name = identities[0].get('name', '')
 		if name:
@@ -1429,32 +1421,32 @@ class ToplevelAgentBrowser(AgentBrowser):
 			break
 
 		# Check if we have to move categories
-		old_cat_iter = model.iter_parent(iter)
-		old_cat = model.get_value(old_cat_iter, 3).decode('utf-8')
-		if model.get_value(old_cat_iter, 3) == cat:
+		old_cat_iter = self.model.iter_parent(iter)
+		old_cat = self.model.get_value(old_cat_iter, 3).decode('utf-8')
+		if self.model.get_value(old_cat_iter, 3) == cat:
 			# Already in the right category, just update
-			model[iter][2] = pix
-			model[iter][3] = descr
-			model[iter][4] = 0
+			self.model[iter][2] = pix
+			self.model[iter][3] = descr
+			self.model[iter][4] = 0
 			return
 		# Not in the right category, move it.
-		model.remove(iter)
+		self.model.remove(iter)
 
 		# Check if the old category is empty
-		if not model.iter_is_valid(old_cat_iter):
+		if not self.model.iter_is_valid(old_cat_iter):
 			old_cat_iter = self._find_category(old_cat)
-		if not model.iter_children(old_cat_iter):
-			model.remove(old_cat_iter)
+		if not self.model.iter_children(old_cat_iter):
+			self.model.remove(old_cat_iter)
 
 		cat_iter = self._find_category(cat, type)
 		if not cat_iter:
 			cat_iter = self._create_category(cat, type)
-		model.append(cat_iter, (jid, node, pix, descr, 0))
+		self.model.append(cat_iter, (jid, node, pix, descr, 0))
 		self._expand_all()
 
-	def _update_error(self, model, iter, jid, node):
+	def _update_error(self, iter, jid, node):
 		addr = get_agent_address(jid, node)
-		model[iter][4] = 2
+		self.model[iter][4] = 2
 		self._progress += 1
 		self._update_progressbar()
 
@@ -1468,11 +1460,13 @@ class MucBrowser(AgentBrowser):
 		# JID, node, name, users, description, fetched
 		# This is rather long, I'd rather not use a data_func here though.
 		# Users is a string, because want to be able to leave it empty.
-		model = gtk.ListStore(str, str, str, str, str, bool)
-		model.set_sort_column_id(2, gtk.SORT_ASCENDING)
-		self.window.services_treeview.set_model(model)
+		self.model = gtk.ListStore(str, str, str, str, str, bool)
+		self.model.set_sort_column_id(2, gtk.SORT_ASCENDING)
+		self.window.services_treeview.set_model(self.model)
 		# Name column
 		col = gtk.TreeViewColumn(_('Name'))
+		col.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+		col.set_fixed_width(100)
 		renderer = gtk.CellRendererText()
 		col.pack_start(renderer)
 		col.set_attributes(renderer, text = 2)
@@ -1492,6 +1486,13 @@ class MucBrowser(AgentBrowser):
 		col.set_attributes(renderer, text = 4)
 		self.window.services_treeview.insert_column(col, -1)
 		col.set_resizable(True)
+		# Id column
+		col = gtk.TreeViewColumn(_('Id'))
+		renderer = gtk.CellRendererText()
+		col.pack_start(renderer)
+		col.set_attributes(renderer, text = 0)
+		self.window.services_treeview.insert_column(col, -1)
+		col.set_resizable(True)
 		self.window.services_treeview.set_headers_visible(True)
 		# Source id for idle callback used to start disco#info queries.
 		self._fetch_source = None
@@ -1501,7 +1502,8 @@ class MucBrowser(AgentBrowser):
 		self.vadj = self.window.services_scrollwin.get_property('vadjustment')
 		self.vadj_cbid = self.vadj.connect('value-changed', self.on_scroll)
 		# And to size changes
-		self.size_cbid = self.window.services_scrollwin.connect('size-allocate', self.on_scroll)
+		self.size_cbid = self.window.services_scrollwin.connect(
+			'size-allocate', self.on_scroll)
 
 	def _clean_treemodel(self):
 		if self.size_cbid:
@@ -1530,14 +1532,13 @@ class MucBrowser(AgentBrowser):
 		if not iter:
 			return
 		service = model[iter][0].decode('utf-8')
-		if service.find('@') != -1:
-			services = service.split('@', 1)
-			room = services[0]
-			service = services[1]
-		else:
-			room = model[iter][1].decode('utf-8')
-		if not gajim.interface.instances[self.account].has_key('join_gc'):
-			dialogs.JoinGroupchatWindow(self.account, service, room)
+		room = model[iter][1].decode('utf-8')
+		if 'join_gc' not in gajim.interface.instances[self.account]:
+			try:
+				room_jid = '%s@%s' % (service, room)
+				dialogs.JoinGroupchatWindow(self.account, service)
+			except GajimGeneralException:
+				pass
 		else:
 			gajim.interface.instances[self.account]['join_gc'].window.present()
 		self.window.destroy(chain = True)
@@ -1571,7 +1572,6 @@ class MucBrowser(AgentBrowser):
 			# Prevent a silly warning, try again in a bit.
 			self._fetch_source = gobject.timeout_add(100, self._start_info_query)
 			return
-		model = view.get_model()
 		# We have to do this in a pygtk <2.8 compatible way :/
 		#start, end = self.window.services_treeview.get_visible_range()
 		rect = view.get_visible_rect()
@@ -1580,7 +1580,7 @@ class MucBrowser(AgentBrowser):
 		try:
 			sx, sy = view.tree_to_widget_coords(rect.x, rect.y)
 			spath = view.get_path_at_pos(sx, sy)[0]
-			iter = model.get_iter(spath)
+			iter = self.model.get_iter(spath)
 		except TypeError:
 			self._fetch_source = None
 			return
@@ -1594,14 +1594,14 @@ class MucBrowser(AgentBrowser):
 		except TypeError:
 			# We're at the end of the model, we can leave end=None though.
 			pass
-		while iter and model.get_path(iter) != end:
-			if not model.get_value(iter, 5):
-				jid = model.get_value(iter, 0).decode('utf-8')
-				node = model.get_value(iter, 1).decode('utf-8')
+		while iter and self.model.get_path(iter) != end:
+			if not self.model.get_value(iter, 5):
+				jid = self.model.get_value(iter, 0).decode('utf-8')
+				node = self.model.get_value(iter, 1).decode('utf-8')
 				self.cache.get_info(jid, node, self._agent_info)
 				self._fetch_source = True
 				return
-			iter = model.iter_next(iter)
+			iter = self.model.iter_next(iter)
 		self._fetch_source = None
 
 	def _channel_altinfo(self, jid, node, items, name = None):
@@ -1622,22 +1622,21 @@ class MucBrowser(AgentBrowser):
 				self._fetch_source = None
 				return
 		else:
-			model = self.window.services_treeview.get_model()
 			iter = self._find_item(jid, node)
 			if iter:
 				if name:
-					model[iter][2] = name
-				model[iter][3] = len(items)		# The number of users
-				model[iter][5] = True
+					self.model[iter][2] = name
+				self.model[iter][3] = len(items)		# The number of users
+				self.model[iter][5] = True
 		self._fetch_source = None
 		self._query_visible()
 
-	def _add_item(self, model, jid, node, item, force):
-		model.append((jid, node, item.get('name', ''), '', '', False))
+	def _add_item(self, jid, node, item, force):
+		self.model.append((jid, node, item.get('name', ''), '', '', False))
 		if not self._fetch_source:
 			self._fetch_source = gobject.idle_add(self._start_info_query)
 
-	def _update_info(self, model, iter, jid, node, identities, features, data):
+	def _update_info(self, iter, jid, node, identities, features, data):
 		name = identities[0].get('name', '')
 		for form in data:
 			typefield = form.getField('FORM_TYPE')
@@ -1647,14 +1646,14 @@ class MucBrowser(AgentBrowser):
 				users = form.getField('muc#roominfo_occupants')
 				descr = form.getField('muc#roominfo_description')
 				if users:
-					model[iter][3] = users.getValue()
+					self.model[iter][3] = users.getValue()
 				if descr:
-					model[iter][4] = descr.getValue()
+					self.model[iter][4] = descr.getValue()
 				# Only set these when we find a form with additional info
 				# Some servers don't support forms and put extra info in
 				# the name attribute, so we preserve it in that case.
-				model[iter][2] = name
-				model[iter][5] = True
+				self.model[iter][2] = name
+				self.model[iter][5] = True
 				break
 		else:
 			# We didn't find a form, switch to alternate query mode
@@ -1664,10 +1663,218 @@ class MucBrowser(AgentBrowser):
 		self._fetch_source = None
 		self._query_visible()
 
-	def _update_error(self, model, iter, jid, node):
+	def _update_error(self, iter, jid, node):
 		# switch to alternate query mode
 		self.cache.get_items(jid, node, self._channel_altinfo)
 
+def PubSubBrowser(account, jid, node):
+	''' Returns an AgentBrowser subclass that will display service discovery
+	for particular pubsub service. Different pubsub services may need to
+	present different data during browsing. '''
+	# for now, only discussion groups are supported...
+	# TODO: check if it has appropriate features to be such kind of service
+	return DiscussionGroupsBrowser(account, jid, node)
+
+class DiscussionGroupsBrowser(AgentBrowser):
+	''' For browsing pubsub-based discussion groups service. '''
+	def __init__(self, account, jid, node):
+		AgentBrowser.__init__(self, account, jid, node)
+
+		# this will become set object when we get subscriptions; None means
+		# we don't know yet which groups are subscribed
+		self.subscriptions = None
+
+		# this will become our action widgets when we create them; None means
+		# we don't have them yet (needed for check in callback)
+		self.subscribe_button = None
+		self.unsubscribe_button = None
+
+		gajim.connections[account].send_pb_subscription_query(jid, self._subscriptionsCB)
+
+	def _create_treemodel(self):
+		''' Create treemodel for the window. '''
+		# JID, node, name (with description) - pango markup, dont have info?, subscribed?
+		model = gtk.ListStore(str, str, str, bool, bool)
+		model.set_sort_column_id(3, gtk.SORT_ASCENDING)
+		self.window.services_treeview.set_model(model)
+
+		# Name column
+		# Pango markup for name and description, description printed with
+		# <small/> font
+		renderer = gtk.CellRendererText()
+		col = gtk.TreeViewColumn(_('Name'))
+		col.pack_start(renderer)
+		col.set_attributes(renderer, markup=2)
+		col.set_resizable(True)
+		self.window.services_treeview.insert_column(col, -1)
+
+		# Subscription state
+		renderer = gtk.CellRendererToggle()
+		col = gtk.TreeViewColumn(_('Subscribed'))
+		col.pack_start(renderer)
+		col.set_attributes(renderer, inconsistent=3, active=4)
+		col.set_resizable(False)
+		self.window.services_treeview.insert_column(col, -1)
+
+		self.window.services_treeview.set_headers_visible(True)
+
+	def _add_item(self, model, jid, node, item, force):
+		''' Called when we got basic information about new node from query.
+		Show the item. '''
+		name = item.get('name', '')
+
+		if self.subscriptions is not None:
+			dunno = False
+			subscribed = name in self.subscriptions
+		else:
+			dunno = True
+			subscribed = False
+
+		name = gtkgui_helpers.escape_for_pango_markup(name)
+		name = '<b>%s</b>' % name
+
+		model.append((jid, node, name, dunno, subscribed))
+
+	def _add_actions(self):
+		self.post_button = gtk.Button(label=_('New post'), use_underline=True)
+		self.post_button.set_sensitive(False)
+		self.post_button.connect('clicked', self.on_post_button_clicked)
+		self.window.action_buttonbox.add(self.post_button)
+		self.post_button.show_all()
+
+		self.subscribe_button = gtk.Button(label=_('_Subscribe'), use_underline=True)
+		self.subscribe_button.set_sensitive(False)
+		self.subscribe_button.connect('clicked', self.on_subscribe_button_clicked)
+		self.window.action_buttonbox.add(self.subscribe_button)
+		self.subscribe_button.show_all()
+
+		self.unsubscribe_button = gtk.Button(label=_('_Unsubscribe'), use_underline=True)
+		self.unsubscribe_button.set_sensitive(False)
+		self.unsubscribe_button.connect('clicked', self.on_unsubscribe_button_clicked)
+		self.window.action_buttonbox.add(self.unsubscribe_button)
+		self.unsubscribe_button.show_all()
+
+	def _clean_actions(self):
+		if self.post_button is not None:
+			self.post_button.destroy()
+			self.post_button = None
+
+		if self.subscribe_button is not None:
+			self.subscribe_button.destroy()
+			self.subscribe_button = None
+
+		if self.unsubscribe_button is not None:
+			self.unsubscribe_button.destroy()
+			self.unsubscribe_button = None
+
+	def update_actions(self):
+		'''Called when user selected a row. Make subscribe/unsubscribe buttons
+		sensitive appropriatelly.'''
+		# we have nothing to do if we don't have buttons...
+		if self.subscribe_button is None: return
+
+		model, iter = self.window.services_treeview.get_selection().get_selected()
+		if not iter or self.subscriptions is None:
+			# no item selected or no subscriptions info, all buttons are insensitive
+			self.post_button.set_sensitive(False)
+			self.subscribe_button.set_sensitive(False)
+			self.unsubscribe_button.set_sensitive(False)
+		else:
+			subscribed = model.get_value(iter, 4) # 4 = subscribed?
+			self.post_button.set_sensitive(subscribed)
+			self.subscribe_button.set_sensitive(not subscribed)
+			self.unsubscribe_button.set_sensitive(subscribed)
+
+	def on_post_button_clicked(self, widget):
+		'''Called when 'post' button is pressed. Open window to create post'''
+		model, iter = self.window.services_treeview.get_selection().get_selected()
+		if iter is None: return
+
+		groupnode = model.get_value(iter, 1)	# 1 = groupnode
+
+		groups.GroupsPostWindow(self.account, self.jid, groupnode)
+
+	def on_subscribe_button_clicked(self, widget):
+		'''Called when 'subscribe' button is pressed. Send subscribtion request.'''
+		model, iter = self.window.services_treeview.get_selection().get_selected()
+		if iter is None: return
+
+		groupnode = model.get_value(iter, 1)	# 1 = groupnode
+
+		gajim.connections[self.account].send_pb_subscribe(self.jid, groupnode, self._subscribeCB, groupnode)
+		
+	def on_unsubscribe_button_clicked(self, widget):
+		'''Called when 'unsubscribe' button is pressed. Send unsubscription request.'''
+		model, iter = self.window.services_treeview.get_selection().get_selected()
+		if iter is None: return
+
+		groupnode = model.get_value(iter, 1)    # 1 = groupnode
+		
+		gajim.connections[self.account].send_pb_unsubscribe(self.jid, groupnode, self._unsubscribeCB, groupnode)
+
+	def _subscriptionsCB(self, conn, request):
+		''' We got the subscribed groups list stanza. Now, if we already
+		have items on the list, we should actualize them. '''
+		print 0
+		try:
+			subscriptions = request.getTag('pubsub').getTag('subscriptions')
+		except:
+			return 
+
+		print 1
+		groups = set()
+		for child in subscriptions.getTags('subscription'):
+			print 2, repr(child), str(child)
+			groups.add(child['node'])
+		print 3, groups
+
+		self.subscriptions = groups
+
+		# try to setup existing items in model
+		model = self.window.services_treeview.get_model()
+		print 4
+		for row in model:
+			print 5
+			# 1 = group node
+			# 3 = insensitive checkbox for subscribed
+			# 4 = subscribed?
+			groupnode = row[1]
+			row[3]=False
+			row[4]=groupnode in groups
+		print 6
+
+		# we now know subscriptions, update button states
+		self.update_actions()
+
+		raise xmpp.NodeProcessed
+
+	def _subscribeCB(self, conn, request, groupnode):
+		'''We have just subscribed to a node. Update UI'''
+		self.subscriptions.add(groupnode)
+
+		model = self.window.services_treeview.get_model()
+		for row in model:
+			if row[1] == groupnode: # 1 = groupnode
+				row[4]=True
+				break
+
+		self.update_actions()
+
+		raise xmpp.NodeProcessed
+
+	def _unsubscribeCB(self, conn, request, groupnode):
+		'''We have just unsubscribed from a node. Update UI'''
+		self.subscriptions.remove(groupnode)
+
+		model = self.window.services_treeview.get_model()
+		for row in model:
+			if row[1] == groupnode: # 1 = groupnode
+				row[4]=False
+				break
+
+		self.update_actions()
+
+		raise xmpp.NodeProcessed
 
 # Fill the global agent type info dictionary
 _agent_type_info = _gen_agent_type_info()
